

#include <FL/Fl.H>
#include <FL/Fl_Double_Window.H>
#include <FL/Fl_Button.H>
#include <FL/Fl_Box.H>
#include <FL/fl_draw.H>
#include <FL/fl_file_chooser.H>
#include <FL/fl_message.H>
#include <FL/filename.H> 
#include "Fl_Cursor_Shape.H"
#include <string.h>
#include <stdio.h>
#include <stdlib.h>


class Canvas : public Fl_Box {
public:
  Canvas(int x, int y, int w, int h) : Fl_Box(x, y, w, h, 0L) { }
  void draw();
  void draw(int, int);
  int handle(int);
};

class TestBox : public Fl_Box {
public:
  TestBox(int x, int y, int w, int h, char *L=0L) : Fl_Box(x, y, w, h, L) {}    
  int handle(int); 
  Fl_Cursor_Shape cursor;
};


Canvas *cv;
char ptr[32][32];
int hotx=0, hoty=0; 
char name[256] = "";
char ptrName[256] = "";

void Canvas::draw(int xp, int yp) {
  char a = ptr[xp][yp];
  if (a==0) fl_color(FL_GRAY);
  else if (a==1) fl_color(FL_WHITE);
  else fl_color(FL_BLACK);
  int xx = x()+xp*8, yy = y()+yp*8;
  fl_rectf(xx+1, yy+1, 7, 7);
  fl_color(FL_DARK2);
  fl_yxline(xx, yy+8, yy, xx+8);
  if (hotx==xp&&hoty==yp) {
    fl_color(FL_RED);
    fl_line(xx+1, yy+1, xx+7, yy+7);
    fl_line(xx+7, yy+1, xx+1, yy+7);
  }
}

void Canvas::draw() {
  int i, j;
  for (i=0; i<32; i++) {
    for (j=0; j<32; j++) {
      draw(i, j);
    }
  }
  fl_color(FL_DARK2);
  fl_xyline(x(), y()+h(), x()+w(), y());
}


int Canvas::handle(int ev) {
  int x, y;
  static int btn = 0;
  static int lookup[] = { 0, 1, 2, 0 };
  switch (ev) {
  case FL_PUSH:
    btn = lookup[Fl::event_button()];
  case FL_DRAG:
    x = (Fl::event_x()-this->x())/8;
    y = (Fl::event_y()-this->y())/8;
    if (x>=0&&x<=32&&y>=0&&y<=32) {
      if (ptr[x][y]!=btn) {
	ptr[x][y] = btn;
	cv->redraw();
      }
      if (Fl::event_state()&FL_SHIFT) { 
	hotx = x; hoty = y; 
	cv->redraw();
      }
    }    
    break;
  }
  return 1;
}

unsigned int toHex(int y, int d) { 
  unsigned int x = 0; 
  int i; 
  for (i=0; i<32; i++) { 
    x = x<<1; 
    if (ptr[i][y]==d) x+=1; 
  } 
  return x;
} 
 
int TestBox::handle( int msg ) 
{
  unsigned int andField[32], xorField[32];
  int i;
  switch ( msg ) 
  {
    case FL_ENTER:
      for ( i=0; i<32; i++ )
      {
        andField[i] = toHex( i, 0 );
        xorField[i] = toHex( i, 1 );
      }
      cursor.shape( hotx, hoty, andField, xorField );
      fl_cursor( &cursor );
      break;
    case FL_LEAVE:
      fl_cursor( FL_CURSOR_DEFAULT );
      break;
  }
  return 1;
}

void clearCB(Fl_Widget*, void*) {
  memset(ptr, 0, 32*32*sizeof(char));
  cv->redraw();
}

void fromHex(const char *s, int y, int d) {
  unsigned int x;
  sscanf(s, "    0x%x", &x);
  int i;
  for (i=0; i<32; i++) {
    if (x&0x80000000) ptr[i][y] += d;
    x = x<<1;
  }
}

void loadCB(Fl_Widget*, void*) {
  char line[257];
  const char *newName = fl_file_chooser("Load:", "*.ptr", name);
  if (!newName) return;
  strcpy(name, newName);
  FILE *in = fopen(name, "rb");
  if (!in) { 
    fl_alert("Can't read file %s!", name);
    return;
  }
  int i;
  memset(ptr, 2, 32*32*sizeof(char));
  fgets(line, 256, in);	// ID
  fgets(line, 256, in);	// uint
  for (i=0; i<32; i++) { fgets(line, 256, in); fromHex(line, i, -2); }
  fgets(line, 256, in);	// }
  fgets(line, 256, in);	// uint
  for (i=0; i<32; i++) { fgets(line, 256, in); fromHex(line, i, -1); }
  fgets(line, 256, in);	// }
  fgets(line, 256, in);	// hot
  const char *a = strchr(line, '=');
  if (a) {
    hotx = atoi(a+1);
    a = strchr(a+1, '=');
    if (a) hoty = atoi(a+1);
  }
  fclose(in);
  cv->redraw();
}

void saveCB(Fl_Widget*, void*) {
  const char *newName = fl_file_chooser("Save:", "*.ptr", name);
  if (!newName) return;
  strcpy(name, newName);
  FILE *out = fopen(name, "wb");
  if (!out) { 
    fl_alert("Can't write file %s!", name);
    return;
  }
  int i;
  const char *a = filename_name(name), *b = filename_ext(a);
  strncpy(ptrName, a, b-a); ptrName[b-a]=0;
  fprintf(out, "/* Matt's crsr format V1.0 */\n");
  fprintf(out, "unsigned int %s_and[32] = {\n", ptrName);
  for (i=0; i<32; i++) 
    fprintf(out, "    0x%08x,\n", toHex(i, 0) );
  fprintf(out, "};\n");
  fprintf(out, "unsigned int %s_xor[32] = {\n", ptrName);
  for (i=0; i<32; i++) 
    fprintf(out, "    0x%08x,\n", toHex(i, 1) ); 
  fprintf(out, "};\n");
  fprintf(out, "int %s_hotX = %d, %s_hotY = %d;\n", ptrName, hotx, ptrName, hoty);
  fclose(out);
}

void main(int argc, char **argv) {
  Fl_Window *win = new Fl_Double_Window(330, 260);
  { Fl_Button *o = new Fl_Button(2, 2, 58, 20, "Load...");
    o->callback(loadCB);
  }
  { Fl_Button *o = new Fl_Button(2, 24, 58, 20, "Save...");
    o->callback(saveCB);
  }
  { Fl_Button *o = new Fl_Button(2, 64, 58, 20, "Clear"); 
    o->callback(clearCB); 
  } 
  TestBox *testBox = new TestBox( 2, 104, 58, 60, "Test" ); 
  testBox->box( FL_THIN_DOWN_BOX );
  cv = new Canvas(64, 2, 257, 257);
  win->end();
  win->show();
  Fl::run();
}
